<?php


namespace MetatickPluginLib\View;

use MetatickPluginLib\Http\HttpResponseNotAllowed;
use MetatickPluginLib\Plugin;
use MetatickPluginLib\Request\Request;

class ViewBase
{
    const INPUT = 0;
    const CHECK = 1;

    protected $template = null;
    protected $template_path = null;
    protected $response = null;
    protected $template_loader = null;
    protected $template_engine = null;

    protected $messages = [];

    public function __construct($template_path)
    {
        $this->template_path = $template_path;
        $this->initialize();
    }

    public function initialize(){

    }

    public function handle_dispatch(){
        $response = $this->dispatch();
        $code = $response->get_code();
        if($code < 200 || $code >= 300){
            $response->send();
        }
    }

    public function dispatch(){
        $method = strtolower($_SERVER['REQUEST_METHOD']);

        if(!method_exists($this, $method)){
            $response =  new HttpResponseNotAllowed($this->get_allowed_methods());
        }else{
            $response = $this->$method();
        }

        if(property_exists ($this, 'slug')) {
            if (count($this->messages) > 0) {
                $response->add_cookie("$this->slug-messages", base64_encode(json_encode($this->messages)), 5);
            } else {
                $request = Request::instance();
                if($request->COOKIES()->get("$this->slug-messages", null) !== null) {
                    $response->delete_cookie("$this->slug-messages");
                }
            }
        }

        $this->response = $response;

        return $response;
    }

    public function get_allowed_methods(){
        $methods = [
            'GET',
            'HEAD',
            'POST',
            'PUT',
            'DELETE',
            'CONNECT',
            'OPTIONS',
            'TRACE',
            'PATCH'
        ];
        $allowed_methods = [];
        foreach($methods as $method){
            if(!method_exists($this, strtolower($method))){
                $allowed_methods[] = $method;
            }
        }
        return $allowed_methods;
    }

    function validate_csrf($name = 'csrf_token'){
        $request = Request::instance();
        $token = $request->POST()->get($name);
        if (!wp_verify_nonce($token)) {
            wp_die('Invalid CSRF Token');
        }
    }

    public function get_template(){
        return $this->template;
    }

    public function get_template_loader(){
        if(!$this->template_loader){
            $this->template_loader = new \Twig\Loader\FilesystemLoader($this->template_path);
        }
        return $this->template_loader;
    }

    public function get_template_engine(){
        if(!$this->template_engine) {
            $template_loader = $this->get_template_loader();
            $this->template_engine = new \Twig\Environment($template_loader);
            $csrf_token = wp_create_nonce(-1);
            $this->template_engine->addGlobal('csrf_token', $csrf_token);

            $csrf_token_function = new \Twig\TwigFunction('csrf_token', function () {
                return wp_nonce_field(-1,'csrf_token',false, false);
            }, ['is_safe' => ['html']]);
            $this->template_engine->addFunction($csrf_token_function);
        }
        return $this->template_engine;
    }

    public function add_message($type, $message){
        $this->messages[] = [$type, $message];
    }

    public function get_messages(){
        return $this->messages;
    }

    public function read_messages(){
        $request = Request::instance();
        $messages = $request->COOKIES()->get("$this->slug-messages");
        try{
            return json_decode(base64_decode($messages));
        }catch (\Exception $e){
            return [];
        }
    }
}